import os
import shutil
import subprocess
import shlex

import jinja2

from .config import Config
from .files import file_class_for_path
from .jinja_extensions import default_extensions


class Combine:
    def __init__(self, config_path, env=None):
        self.config_path = config_path
        self.env = env
        self.load()

    def load(self):
        self.config = Config(self.config_path)
        self.output_path = self.config.output_path
        self.content_paths = self.config.content_paths

        self.content_directories = [ContentDirectory(x) for x in self.content_paths]

        choice_loaders = [
            jinja2.FileSystemLoader(x.path) for x in self.content_directories
        ]

        self.jinja_environment = jinja2.Environment(
            loader=jinja2.ChoiceLoader(choice_loaders),
            autoescape=jinja2.select_autoescape(["html", "xml"]),
            undefined=jinja2.StrictUndefined,  # make sure variables exist
            extensions=default_extensions,
        )
        self.jinja_environment.globals.update(self.get_jinja_variables())

    def get_jinja_variables(self):
        variables = self.config.variables
        variables["env"] = self.env
        return variables

    def reload(self):
        """Reload the config and entire jinja environment"""
        self.load()

    def clean(self):
        if os.path.exists(self.output_path):
            shutil.rmtree(self.output_path)

    def build(self, only_paths=None):
        if not only_paths:
            # completely wipe it
            self.clean()

        if not os.path.exists(self.output_path):
            os.mkdir(self.output_path)

        paths_rendered = []

        for content_directory in self.content_directories:
            for file in content_directory.files:
                if (
                    file.output_relative_path
                    and file.output_relative_path not in paths_rendered
                ):
                    if only_paths and file.path not in only_paths:
                        continue

                    file.render_to_output(
                        self.output_path, jinja_environment=self.jinja_environment
                    )
                    paths_rendered.append(file.output_relative_path)

        # If building the entire site, run the custom steps now
        if not only_paths:
            for step in self.config.steps:
                subprocess.run(shlex.split(step["run"]), check=True)

    def get_file_obj_for_path(self, path):
        for content_directory in self.content_directories:
            for file in content_directory.files:
                if os.path.abspath(file.path) == os.path.abspath(path):
                    return file

        return None

    def is_in_content_paths(self, path):
        for content_path in self.content_paths:
            if (
                os.path.commonpath([content_path, path]) != os.getcwd()
                and os.getcwd() in content_path
            ):
                return True
        return False

    def is_in_output_path(self, path):
        return (
            os.path.commonpath([self.output_path, os.path.abspath(path)]) != os.getcwd()
        )


class ContentDirectory:
    def __init__(self, path):
        assert os.path.exists(path), f"Path does not exist: {path}"
        self.path = path
        self.load_files()

    def load_files(self):
        self.files = []

        for root, dirs, files in os.walk(self.path, followlinks=True):
            for file in files:
                file_path = os.path.join(root, file)
                self.files.append(file_class_for_path(file_path)(file_path, self))

    def file_classes(self):
        return set([x.__class__ for x in self.files])



import os

import frontmatter

from .html import HTMLFile
from .utils import create_parent_directory


class MarkdownFile(HTMLFile):
    def render_to_output(self, output_path, *args, **kwargs):
        post = frontmatter.load(self.path)

        variables = post.metadata
        variables["url"] = self._get_url()
        variables["content"] = post.content

        template = kwargs["jinja_environment"].get_template("markdown.template.html")

        target_path = os.path.join(output_path, self.output_relative_path)
        create_parent_directory(target_path)

        with open(target_path, "w+") as f:
            f.write(template.render(**variables))



from setuptools import setup, find_packages


with open('README.md') as f:
    long_description = f.read()

requirements = (
    'jinja2',
    'click',
    'watchdog',
    'pyyaml',
    'beautifulsoup4',
    'Pygments',
    'markdown',
)

setup(
    name='combine',
    version='0.0.23',
    description='A helpful, simple static site generator.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Dropseed',
    author_email='python@dropseed.io',
    python_requires='>=3.6.0',
    url='https://github.com/dropseed/combine',
    packages=find_packages(exclude=('tests', 'docs')),
    entry_points={
        'console_scripts': ['combine=combine.cli:cli'],
    },
    install_requires=requirements,
    include_package_data=True,
    license='MIT',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],
)



import os

from .core import File
from .ignored import IgnoredFile
from .template import TemplateFile
from .html import HTMLFile
from .markdown import MarkdownFile
from .redirect import RedirectFile


def file_class_for_path(path):
    base, ext = os.path.splitext(path)

    if os.path.basename(path).startswith("_") or os.path.basename(path).startswith("."):
        return IgnoredFile

    secondary_extension_classes = {".template": TemplateFile}

    base2, ext2 = os.path.splitext(base)

    if ext2 in secondary_extension_classes:
        return secondary_extension_classes[ext2]

    classes = {".html": HTMLFile, ".redirect": RedirectFile, ".md": MarkdownFile}

    if ext in classes:
        return classes[ext]

    return File
